package de.ozgcloud.admin.setting;

import java.util.Map;
import java.util.Optional;

import org.apache.commons.lang3.StringUtils;
import org.springframework.cloud.config.environment.Environment;
import org.springframework.cloud.config.environment.PropertySource;
import org.springframework.cloud.config.server.environment.EnvironmentRepository;
import org.springframework.stereotype.Component;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;

import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
@Component
public class SettingEnvironmentRepository implements EnvironmentRepository {

	private final SettingService settingService;

	private final ObjectMapper objectMapper;

	private static final String ALFA = "Alfa";
	private static final String VORGANG_MANAGER = "OzgCloud_VorgangManager";

	@Override
	public Environment findOne(String application, String profile, String label) {
		return buildEnvironment(application, findAnwendungSettingDTO(application));
	}

	Optional<ApplicationSettingDTO> findAnwendungSettingDTO(String application) {
		switch (application) {
			case ALFA:
				return Optional.of(settingService.getAlfaSettingDTO());
			case VORGANG_MANAGER:
				return Optional.of(settingService.getVorgangManagerSettingDTO());
			default:
				return Optional.empty();
		}
	}

	Environment buildEnvironment(String application, Optional<ApplicationSettingDTO> settingDTO) {
		var environment = new Environment(application);
		settingDTO.ifPresent(setting -> environment.add(transformToPropertySource(setting)));
		return environment;
	}

	PropertySource transformToPropertySource(ApplicationSettingDTO settingDTO) {
		return new PropertySource(StringUtils.EMPTY, objectMapper.convertValue(settingDTO, new TypeReference<Map<String, Object>>() {
		}));
	}

}
